<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	error_reporting(E_ALL); ini_set("display_errors", 1);
	ini_set("soap.wsdl_cache_enabled", "1");
	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require("VOMSSOAP.php");

	// WSDL files
	$wsdl_file = 'wsdl/voms-admin-2.0.2.wsdl';
	$wsdl_server_file = 'wsdl/voms-admin-2.0.2-server.wsdl';

	// WSDL URL
	$wsdl_url = 'http://'.$_SERVER['HTTP_HOST'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']);

	// return WSDL
	if ( isset($_GET['wsdl']) ) {
		$wsdl = file_get_contents($wsdl_file);
		echo preg_replace('/@@ENDPOINT@@/',$wsdl_url,$wsdl);
		exit(0);
	}

	// define handler class
	class VOMSAdmin extends VOMSSOAP {
		/*
		 * USER MANAGEMENT COMMANDS
		 */
		public function createUser($u) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "membership", "w");
			// generate CN if not specified
			$user_cn = ( $u->CN !== NULL ) ? $u->CN : CNfromDN($u->DN);
			// create user inside transaction
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'createUser', 
					$u->DN, $u->CA, $user_cn, $u->mail, $this->vo);
			if ( $result === -11 ) {
				$this->request_failed ("Server.userException",
					sprintf("UserAlreadyExistsException: User (%s,%s) already exists in database!",
					$u->DN, $u->CA));
			} else if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					"UntrustedCAException: User have untrusted CA. Contact VOMS server admin about new CA support.");
			} else return $result;
		}

		public function deleteUser($dn, $ca) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "membership", "w");
			// get user ID
			$uid = $this->get_uid($dn, $ca);
			// delete user
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'deleteUser',$uid);
		}

		// NOTE: getUser and setUser does not used by voms-admin-clients 
		public function getUser($dn, $ca) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "membership", "r");
			// get user ID
			$uid = $this->get_uid($dn, $ca);
			// get user Info
			$uinfo = getUserInfo($uid);
			return   array(	"CA" => $uinfo[2],
					"CN" => $uinfo[1],
					"DN" => $uinfo[0],
					"certUri" =>$uinfo[5],
					"mail" =>$uinfo[3] );
		}

		public function setUser($u) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "membership", "w");
			// get user ID
			$uid = $this->get_uid($u->DN, $u->CA);
			// update user info
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'updateUserInfo', $uid, $u->CN, $u->mail);	
		}

		/*
		 * ROLE ASSIGNMENT COMMANDS
		 */
		public function assignRole($group_name, $role_name, $dn, $ca) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			if ( $role_name !== NULL ) {
				$rid = $this->get_rid($role_name);
			} else $rid = 0;
			// get user ID
			$uid = $this->get_uid($dn, $ca);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "membership", "w" );
			// add user to group
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'addMembership',$gid,$uid,$rid);
			if ( $result === -1 ) {
				if ( $rid ) $rolemsg = sprintf(" with %s", $role_name); else $rolemsg = "";
				$this->request_failed ("Server.userException",
					sprintf("UserAlreadyMemberException: User %s,%s is already a member of '%s' group%s",$dn,$ca,$group_name,$rolemsg));
			} else return $result;
		}

		public function dismissRole($group_name, $role_name, $dn, $ca) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			if ( $role_name !== NULL ) {
				$rid = $this->get_rid($role_name);
			} else $rid = 0;
			// get user ID
			$uid = $this->get_uid($dn, $ca);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "membership", "w" );
			// delete user from group
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'delMembership', $gid, $uid, $rid);
			if ( $result === -1 ) {
				if ( $rid ) $rolemsg = sprintf(" with %s", $role_name); else $rolemsg = "";
				$this->request_failed ("Server.userException",
					sprintf("UserNotMemberException: User %s,%s is not a member of '%s' group%s",$dn,$ca,$group_name,$rolemsg));
			} else return $result;
		}
	
		public function listUsersWithRole($group_name, $role_name) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// get role ID
			$rid = $this->get_rid($role_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "membership", "r" );
			// get group members list
			return getAllUsersInfo($gid, $rid);
		}

		/*
		 * ROLE MANAGEMENT COMMANDS
		 */
		public function listRoles ($dn = NULL, $ca = NULL) {
			// get roles list
			$roles = array ();
			if ( $dn || $ca ) {
				// get user ID
				$uid = $this->get_uid($dn, $ca);
				// get user roles
				$ummb = getUserMembership($uid);
				if ( ! empty($ummb) ) {
					$allgroups = getGroups(-1);
					foreach ( $ummb as $mg => $mr ) {
						// check permissions
						$mgid = $allgroups[$mg];
						if ( ! isACLallow(get_ACL_permissions($mgid),"container","r",1) ) continue;
						// get assigned roles in group
						foreach ( $mr as $rname ) {
							if ( empty($rname) ) continue;
							$roles[] = $mg . "/Role=" . $rname;
						}
					}
				}
			} else {
				// check permissions
				$this->acl_check(get_ACL_permissions(), "container","r");
				// get all defined roles
				$roles_hash = getRoles(-1);
				foreach ( array_keys($roles_hash) as $rname ) {
					$roles[] = "Role=" . $rname;
				}
			}
			return $roles;
		}

		public function createRole($rname) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "container", "w");
			// remove possible 'Role=' prefix from name
			$rname = preg_replace(array('/Role=/','/ROLE=/'), array('',''), $rname);
			// create role
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'createRole', $rname, $this->vo );
			if ( $result === -1  ) {
				$this->request_failed ("Server.userException",
					sprintf("RoleAlreadyExistsException: Role '%s' already exists in database!", $rname));
			} else return $result;
		}

		public function deleteRole($rname) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "container", "w");
			// remove possible 'Role=' prefix from name
			$rname = preg_replace(array('/Role=/','/ROLE=/'), array('',''), $rname);
			// get role id
			$rid = getRoleId($rname);
			if ( ! $rid ) $this->request_failed("Server.userException",
				"NoSuchRoleException: Role '".$rname."' is not defined for this vo.");
			// delete role
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'deleteRole',$rid, $this->vo);
		}

		/*
		 * GROUP MEMBERSHIP MANAGEMENT COMMANDS
		 */
		public function listMembers($group_name = ""){
			 // check group name
			$this->parse_group_name($group_name, true);
			// get group ID
			if ( $group_name ) {
				$gid = $this->get_gid($group_name);
			} else $gid = NULL; 
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "membership", "r");
			// get group members list
			$result = getAllUsersInfo($gid);
			if ( ! $result ) $this->request_failed("Server.userException",
				sprintf("NoGroupMembersException: Group '%s' does not contains members.",$group_name));
			return $result;
		}

		public function addMember($group_name, $dn, $ca) {
			return $this->assignRole($group_name, NULL, $dn, $ca);
		}

		public function removeMember($group_name, $dn, $ca) {
			return $this->dismissRole($group_name, NULL, $dn, $ca);
		}


		/*
		 * GROUP MANAGEMENT COMMANDS
		 */
		public function getVOName() {
			return "/".$this->vo;
		}

		public function listSubGroups($group_name) {
			// get group id
			if ( $group_name ) {
				$gid = $this->get_gid($group_name);
			} else $gid = 1;
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "container", "r");
			// list subgroups
			$group_list = getGroups(-1,null,$gid);
			//$this->request_failed ("Server.Debug", "Server.Debug: agrument dump" . var_export($group_list, true) ); 
			return array_keys($group_list);
		}

		public function listGroups ($dn, $ca) {
			$groups = array ();
			// get user ID
			$uid = $this->get_uid($dn, $ca);
			// get user groups
			$ummb = getUserMembership($uid);
			if ( ! empty($ummb) ) {
				$allgroups = getGroups(-1);
				foreach ( $ummb as $mg => $mr ) {
					// check permissions
					$mgid = $allgroups[$mg];
					if ( isACLallow(get_ACL_permissions($mgid),"container","r",1) ) {
						$groups[] = $mg;
					}
				}
			}
			return $groups;
		}
	
		public function createGroup($unknown, $groupname) {
			// check group name
			$gname_arr = $this->parse_group_name($groupname);
			// get new group and parent
			$newgroup = array_pop($gname_arr); 
			$gparent = implode('/',$gname_arr);
			$gparent_gid = $this->get_gid($gparent);
			// check permissions for parent group
			$this->acl_check(get_ACL_permissions($gparent_gid),"container","w");
			// create group (name,parentgroupid,vo)
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'createGroup', $newgroup, $gparent_gid, $this->vo);
			if ( $result === -1  ) {
				$this->request_failed ("Server.userException",
					sprintf("GroupAlreadyExistsException: Group '%s' already exists in database!", $groupname));
			} else return $result;
		}

		public function deleteGroup($groupname) {
			// check group name
			$gname_arr = $this->parse_group_name($groupname);
			// get group ID
			$gid =  $this->get_gid($groupname);
			// get parent group
			$newgroup = array_pop($gname_arr); 
			$gparent = implode('/',$gname_arr);
			$gparent_gid = $this->get_gid($gparent);
			// check permissions for parent group
			$this->acl_check(get_ACL_permissions($gparent_gid),"container","w");
			// delete group
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'deleteGroup', $gid, $this->vo);
			if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					sprintf("GroupHasChildException: Group '%s' cannot be deleted since it contains subgroups.", $groupname));
			} else return $result;
		}

		/*
		 * CONTAINER OPERATIONS (OBSOLETE, NOT SUPPORTED)
		 */
		private function capabilityException () {
			$this->request_failed ("Server.vomsException",
				"Capabilities are now obsolete and does not supported by PHP VOMS-Admin. You can use attributes instead.");
		}

		public function createCapability ($in0){
			$this->capabilityException();
		}

		public function deleteCapability ($in0){
			$this->capabilityException();
		}

		public function assignCapability ($in0, $in1, $in2) {
			$this->capabilityException();
		}

		public function dissmissCapability ($in0, $in1, $in2) {
			$this->capabilityException();
		}

		public function listUsersWithCapability ($in0) {
			$this->capabilityException();
		}

		public function listCapabilities ($in0, $in1) {
			$this->capabilityException();
		}

		/*
		 * ACL OPERATIONS (OBSOLETE, USE VOMSACL INTERFACE INSTEAD)
		 */
		private function useaclException () {
			$this->request_failed ("Server.vomsException",
				"ACL operations via VOMSAdmin SOAP interface are now obsolete. Use VOMSACL interface instead.");
		}

		public function getACL ($in0){
			$this->useaclException();
		}

		public function setACL ($in0, $in1) {
			$this->useaclException();
		}

		public function addACLEntry ($in0, $in1) {
			$this->useaclException();
		}

		public function removeACLEntry ($in0, $in1) {
			$this->useaclException();
		}

		public function getDefaultACL ($in0) {
			$this->useaclException();
		}

		public function setDefaultACL ($in0, $in1) {
			$this->useaclException();
		}

		public function addDefaultACLEntry ($in0, $in1) {
			$this->useaclException();
		}

		public function removeDefaultACLEntry ($in0, $in1) {	
			$this->useaclException();
		}

		/*
		 * OTHER COMMANDS
		 */
		public function listCAs () {
			return getCAList();
		}
		
		public function getGroupPath ($container) {
			// NOTE: functionality does not depend on VO groups structure, just remote parser
			// 	method does not actually used by voms-admin-clients but exists in WSDL
			$pcont = $this->parse_container($container);
			// create group path
			$result = array ( $container );
			$gp = '';
			foreach ( $pcont["groups"] as $g ) {
				$gp .= '/' . $g;
				$result[] = $gp;
			}
			array_pop($result);
			return $result;
		}
	}

	// create SAOP server object
	$server = new SoapServer($wsdl_server_file, array(
			'soap_version' => SOAP_1_1 ));
	$server->setClass('VOMSAdmin',$wsdl_file);

	// handle client request
	$data = VOMSSOAP::getSOAPRequestData();
	if ( ! $data ) $server->fault('ns1:Client',"Error: method not specified");
	
	// overcome to handle calls with and without parameters for previously overloaded methods
	$regex_empty_req = '/<([^>\/ ]*)(listMembers|listRoles|listCapabilities)([^>\/]*)(\/>|><\/([^>]+)>)/';
	$subst_empty_req = '<\1\2\3><in0></in0><in1></in1></\1\2>';
	$data = preg_replace($regex_empty_req, $subst_empty_req, $data);

	// catching SOAPFault exceptions does not work for some reson
	// here the hack to return proper exceptions output
	ob_start();
	$server->handle($data);
	$soapResponse = ob_get_contents();
	ob_end_clean();

	header('HTTP/1.0 200 OK');
	echo $soapResponse;
?>
