<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	error_reporting(E_ALL); ini_set("display_errors", 1);
	ini_set("soap.wsdl_cache_enabled", "1");
	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require("VOMSSOAP.php");

	// WSDL files
	$wsdl_file = 'wsdl/voms-attributes-2.0.2.wsdl';
	$wsdl_server_file = 'wsdl/voms-attributes-2.0.2-server.wsdl';

	// WSDL URL
	$wsdl_url = 'http://'.$_SERVER['HTTP_HOST'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']);

	// return WSDL
	if ( isset($_GET['wsdl']) ) {
		$wsdl = file_get_contents($wsdl_file);
		echo preg_replace('/@@ENDPOINT@@/',$wsdl_url,$wsdl);
		exit(0);
	}

	// define handler class
	class VOMSAttributes extends VOMSSOAP {
		private function AttributeClass($name, $descr = null, $uniq = false) {
			if ( ! $descr ) $descr = null;
			return array (
				"name" => $name,
				"description" => $descr,
				"uniquenessChecked" => $uniq
			);
		}
		private function AttributeValue($attrclass, $value) {
			return array (
				"attributeClass" => $attrclass,
				"context" => null,
				"value" => $value
			);
		}
		
		public function createAttributeClass($classname, $description = '', $unique = false) {
			// check name syntax
			$this->name_check($classname);
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "w" );
			// create attribute class
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'addVOAttribute',$classname,$description,$unique);
			if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					sprintf("AttributeClassAlreadyExistsException: Attribute class '%s' already exists in database!", 
					$classname));
			} else return $result;
                }


		public function deleteAttributeClass($in0) {
			// handle AttributeClass vs soapenc:string
			$classname = $this->getSOAPOverloadedParameter($in0, 'name', 'deleteAttributeClass');
			// check name syntax
			$this->name_check($classname);
			// get attribute ID
			$aid = $this->get_aid($classname);
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "w" );
			// delete attribute class
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'delVOAttribute', $aid);
		}

		public function listAttributeClasses() {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "r" );
			// get attribute classes from DB and construct response
			$result = array();
			foreach ( getAttributes() as $attr_params ) {
				$result[] = $this->AttributeClass($attr_params["name"], $attr_params["descr"], $attr_params["uniq"]);
			}
			return $result;
		}

		public function saveAttributeClass ($cobj) {
			return $this->createAttributeClass($cobj->name, $cobj->description, $cobj->uniquenessChecked);
		}

		public function getAttributeClass ($class_name) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "r" );
			// check name syntax
			$this->name_check($classname);
			// get attribute ID
			$aid = $this->get_aid($classname);
			// get attributes
			$attr_arr = getAttributes();
			$attrclass = $attr_arr[$aid];
			return $this->AttributeClass($attrclass["name"], $attrclass["descr"], $attrclass["uniq"]);
		}

		public function setUserAttribute($user, $attrvalue) {
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "w" );
			// get user ID
			$uid = $this->get_uid($user->DN, $user->CA);
			// check name syntax
			$this->name_check($attrvalue->attributeClass->name);
			// get attribute class ID
			$aid = $this->get_aid($attrvalue->attributeClass->name);
			// invoke attribute set
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'addAttribute', $aid, $uid, $attrvalue->value, 'u');
			if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					sprintf("UniqueAttributeException: Value '%s' already exists for unique attribute '%s'.", 
					$attrvalue->value, $attrvalue->attributeClass->name));
			} else return $result;
		}

		public function deleteUserAttribute($user, $in1) {
			// handle AttributeValue vs soapenc:string
			$attrname = $this->getSOAPOverloadedParameter($in1, 
				array('attributeClass','name'), 'deleteUserAttribute');
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "w" );
			// get user ID
			$uid = $this->get_uid($user->DN, $user->CA);
			// check name syntax
			$this->name_check($attrname);
			// get attribute class ID
			$aid = $this->get_aid($attrname);
			// invoke attribute deletion
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'delAttribute', $aid, $uid, 'u');
		}

		public function setGroupAttribute($group_name, $attrvalue) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "w" );
			// check name syntax
			$this->name_check($attrvalue->attributeClass->name);
			// get attribute class ID
			$aid = $this->get_aid($attrvalue->attributeClass->name);
			// invoke attribute set
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'addAttribute', $aid, $gid, $attrvalue->value, 'g');
			if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					sprintf("UniqueAttributeException: Value '%s' already exists for unique attribute '%s'.", 
					$attrvalue->value, $attrvalue->attributeClass->name));
			} else return $result;
		}

		public function deleteGroupAttribute($group_name, $in1) {
			// handle AttributeValue vs soapenc:string
			$attrname = $this->getSOAPOverloadedParameter($in1, 
				array('attributeClass','name'), 'deleteGroupAttribute');
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "w" );
			// check name syntax
			$this->name_check($attrname);
			// get attribute class ID
			$aid = $this->get_aid($attrname);
			// invoke attribute deletion
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'delAttribute', $aid, $gid, 'g');
		}

		public function setRoleAttribute($group_name, $role_name, $attrvalue) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "w" );
			// get role ID
			$rid = $this->get_rid($role_name);
			// check name syntax
			$this->name_check($attrvalue->attributeClass->name);
			// get attribute class ID
			$aid = $this->get_aid($attrvalue->attributeClass->name);
			// invoke attribute set
			$result = _invoke_transactional_sql(SOAP_UPDATOR_ID,'addRoleAttribute', $aid, $rid, $gid, $attrvalue->value);
			if ( $result === -1 ) {
				$this->request_failed ("Server.userException",
					sprintf("UniqueAttributeException: Value '%s' already exists for unique attribute '%s'.", 
					$attrvalue->value, $attrvalue->attributeClass->name));
			} else return $result;
		}

		public function deleteRoleAttribute($group_name, $role_name, $in1) {
			// handle AttributeValue vs soapenc:string
			$attrname = $this->getSOAPOverloadedParameter($in1, 
				array('attributeClass','name'), 'deleteRoleAttribute');
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "w" );
			// get role ID
			$rid = $this->get_rid($role_name);
			// check name syntax
			$this->name_check($attrname);
			// get attribute class ID
			$aid = $this->get_aid($attrname);
			// invoke attribute deletion
			return _invoke_transactional_sql(SOAP_UPDATOR_ID,'delRoleAttribute', $aid, $rid, $gid);
		}

		public function listUserAttributes($user){
			// check permissions
			$this->acl_check(get_ACL_permissions(), "attributes", "r" );
			// get user ID
			$uid = $this->get_uid($user->DN, $user->CA);
			// construct ArrayOfAttributeValue
			$result = array ();
			foreach ( getUserAttributes($uid) as $atr ) {
				$result[] = $this->AttributeValue(
					$this->AttributeClass($atr["name"], $atr["descr"], $atr["uniq"]),
					$atr["value"]
				);
			}
			return $result;
		}

		public function listGroupAttributes($group_name) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "r" );
			// construct ArrayOfAttributeValue
			$result = array ();
			foreach ( getGroupAttributes($gid) as $atr ) {
				$result[] = $this->AttributeValue(
					$this->AttributeClass($atr["name"], $atr["descr"], $atr["uniq"]),
					$atr["value"]
				);
			}
			return $result;
		}

		public function listRoleAttributes($group_name, $role_name) {
			// check group name
			$this->parse_group_name($group_name);
			// get group ID
			$gid = $this->get_gid($group_name);
			// check permissions
			$this->acl_check(get_ACL_permissions($gid), "attributes", "w" );
			// get role ID
			$rid = $this->get_rid($role_name);
			// construct ArrayOfAttributeValue
			$result = array ();
			foreach ( getRoleAttributes($rid) as $atr ) {
				if ( $atr["groupid"] == $gid ) {
					$result[] = $this->AttributeValue(
						$this->AttributeClass($atr["name"], $atr["descr"], $atr["uniq"]),
						$atr["value"]
					);
				}
			}
			return $result;
		}

	}

	// create SAOP server object
	$server = new SoapServer($wsdl_server_file, array( 
			'soap_version' => SOAP_1_1 )); 
	$server->setClass('VOMSAttributes',$wsdl_file);

	// handle client request
	$data = VOMSSOAP::getSOAPRequestData();
	if ( ! $data ) $server->fault('ns1:Client',"Error: method not specified");
	
	// catching SOAPFault exceptions does not work for some reson at least in Nginx
	// here the hack to return proper exceptions output
	ob_start();
	$server->handle($data);
	$soapResponse = ob_get_contents();
	ob_end_clean();

	header('HTTP/1.0 200 OK');
	echo $soapResponse;
?>
